@@ -1,7 +1,7 @@
 """"
 A predictor for STOI - Short-Time Objective Intelligibility (STOI)
 """
-from nkululeko.util import Util
+from nkululeko.utils.util import Util
 import ast
 import nkululeko.glob_conf as glob_conf
 from nkululeko.feature_extractor import FeatureExtractor
@@ -22,9 +22,7 @@ class STOIPredictor:
     def predict(self, split_selection):
         self.util.debug(f"estimating STOI for {split_selection} samples")
         return_df = self.df.copy()
-        feats_name = "_".join(
-            ast.literal_eval(glob_conf.config["DATA"]["databases"])
-        )
+        feats_name = "_".join(ast.literal_eval(glob_conf.config["DATA"]["databases"]))
         self.feature_extractor = FeatureExtractor(
             self.df, ["squim"], feats_name, split_selection
         )
