@@ -3,19 +3,18 @@ import ast
 import os
 import os.path
 from random import sample
+import numpy as np
+import pandas as pd
 
 import audformat
+from audformat.utils import duration
+
 import nkululeko.filter_data as filter
 import nkululeko.glob_conf as glob_conf
-
-# import audb
-import pandas as pd
-from audformat.utils import duration
 from nkululeko.filter_data import DataFilter
 from nkululeko.plots import Plots
 from nkululeko.reporting.report_item import ReportItem
-from nkululeko.util import Util
-import nkululeko.split.split_utils as split
+from nkululeko.utils.util import Util
 
 
 class Dataset:
@@ -69,7 +68,7 @@ class Dataset:
         try:
             self.db = audformat.Database.load(root)
         except FileNotFoundError:
-            self.util.error(f"{self.name}: no database found at {root}")
+            self.util.error(f"{self.name}: no audformat database found at {root}")
         return root
 
     def _check_cols(self, df):
@@ -77,6 +76,7 @@ class Dataset:
         if rename_cols:
             col_dict = ast.literal_eval(rename_cols)
             df = df.rename(columns=col_dict)
+            self.util.debug(f"renamed data columns: {col_dict}")
         return df
 
     def _report_load(self):
@@ -97,11 +97,12 @@ class Dataset:
         """Load the dataframe with files, speakers and task labels"""
         # store the dataframe
         store = self.util.get_path("store")
-        store_file = f"{store}{self.name}.pkl"
+        store_format = self.util.config_val("FEATS", "store_format", "pkl")
+        store_file = f"{store}{self.name}.{store_format}"
         self.root = self._load_db()
         if not self.start_fresh and os.path.isfile(store_file):
             self.util.debug(f"{self.name}: reusing previously stored file {store_file}")
-            self.df = pd.read_pickle(store_file)
+            self.df = self.util.get_store(store_file, store_format)
             self.is_labeled = self.target in self.df
             self.got_gender = "gender" in self.df
             self.got_age = "age" in self.df
@@ -149,6 +150,13 @@ class Dataset:
                 self.got_speaker = got_speaker2 or self.got_speaker
                 self.got_gender = got_gender2 or self.got_gender
                 self.got_age = got_age2 or self.got_age
+                if audformat.is_filewise_index(df_target.index):
+                    try:
+                        df_target = df_target.loc[df.index.get_level_values("file")]
+                        df_target = df_target.set_index(df.index)
+                    except KeyError:
+                        # just a try...
+                        pass
                 if got_target2:
                     df[self.target] = df_target[self.target]
                 if got_speaker2:
@@ -156,9 +164,16 @@ class Dataset:
                 if got_gender2:
                     df["gender"] = df_target["gender"]
                 if got_age2:
-                    df["age"] = df_target["age"]
+                    df["age"] = df_target["age"].astype(int)
+                # copy other column
+                for column in df_target.columns:
+                    if column not in [self.target, "age", "speaker", "gender"]:
+                        df[column] = df_target[column]
             except audformat.core.errors.BadKeyError:
-                pass
+                if not self.is_labeled:
+                    self.util.error(
+                        f"Giving up: no target ({self.target}) column found"
+                    )
 
         if self.is_labeled:
             # remember the target in case they get labelencoded later
@@ -171,12 +186,16 @@ class Dataset:
         # ensure segmented index
         self.df = self.util.make_segmented_index(self.df)
         self.util.copy_flags(self, self.df)
+        # check the type of numeric targets
+        if not self.util.exp_is_classification():
+            self.df[self.target] = self.df[self.target].astype(float)
         # add duration
         if "duration" not in self.df:
             start = self.df.index.get_level_values(1)
             end = self.df.index.get_level_values(2)
             self.df["duration"] = (end - start).total_seconds()
-
+        elif self.df.duration.dtype == "timedelta64[ns]":
+            self.df["duration"] = self.df["duration"].map(lambda x: x.total_seconds())
         # Perform some filtering if desired
         required = self.util.config_val_data(self.name, "required", False)
         if required:
@@ -195,12 +214,43 @@ class Dataset:
             self.name, "rename_speakers", False
         ):
             # we might need to append the database name to all speakers in case other databases have the same speaker names
+            self.df.speaker = self.df.speaker.astype(str)
             self.df.speaker = self.df.speaker.apply(lambda x: self.name + x)
 
+        # check if the target variable should be reversed
+        def reverse_array(d, max):
+            d = np.array(d)
+            res = []
+            for n in d:
+                res.append(abs(n - max))
+            return res
+
+        reverse = eval(self.util.config_val_data(self.name, "reverse", "False"))
+        if reverse:
+            max = eval(self.util.config_val_data(self.name, "reverse.max", "False"))
+            if max:
+                max = float(max)
+            else:
+                max = self.df[self.target].values.max()
+            self.util.debug(f"reversing target numbers with max values: {max}")
+            self.df[self.target] = reverse_array(self.df[self.target].values, max)
+
+        # check if the target variable should be scaled (z-transformed)
+        scale = self.util.config_val_data(self.name, "scale", False)
+        if scale:
+            from sklearn.preprocessing import StandardScaler
+
+            self.util.debug("scaling target variable to normal distribution")
+            scaler = StandardScaler()
+            self.df[self.target] = scaler.fit_transform(
+                self.df[self.target].values.reshape(-1, 1)
+            )
+
         # store the dataframe
         store = self.util.get_path("store")
-        store_file = f"{store}{self.name}.pkl"
-        self.df.to_pickle(store_file)
+        store_format = self.util.config_val("FEATS", "store_format", "pkl")
+        store_file = f"{store}{self.name}.{store_format}"
+        self.util.write_store(self.df, store_file, store_format)
 
     def _get_df_for_lists(self, db, df_files):
         is_labeled, got_speaker, got_gender, got_age = (
@@ -212,6 +262,7 @@ class Dataset:
         df = pd.DataFrame()
         for table in df_files:
             source_df = db.tables[table].df
+            # check if columns should be renamed
             source_df = self._check_cols(source_df)
             # create a dataframe with the index (the filenames)
             df_local = pd.DataFrame(index=source_df.index)
@@ -232,24 +283,30 @@ class Dataset:
                 # try to get the gender values
                 if "gender" in source_df:
                     df_local["gender"] = source_df["gender"]
-                got_gender = True
+                    got_gender = True
             except (KeyError, ValueError, audformat.errors.BadKeyError) as e:
                 pass
             try:
                 # try to get the age values
-                df_local["age"] = source_df["age"]
+                df_local["age"] = source_df["age"].astype(int)
                 got_age = True
-            except (KeyError, ValueError, audformat.errors.BadKeyError) as e:
+            except (KeyError, ValueError, audformat.errors.BadKeyError):
                 pass
             try:
                 # also it might be possible that the sex is part of the speaker description
                 df_local["gender"] = db[table]["speaker"].get(map="gender")
                 got_gender = True
-            except (ValueError, audformat.errors.BadKeyError) as e:
+            except (ValueError, audformat.errors.BadKeyError):
+                pass
+            try:
+                # also it might be possible that the sex is part of the speaker description
+                df_local["gender"] = db[table]["speaker"].get(map="sex")
+                got_gender = True
+            except (ValueError, audformat.errors.BadKeyError):
                 pass
             try:
                 # also it might be possible that the age is part of the speaker description
-                df_local["age"] = db[table]["speaker"].get(map="age")
+                df_local["age"] = db[table]["speaker"].get(map="age").astype(int)
                 got_age = True
             except (ValueError, audformat.errors.BadKeyError) as e:
                 pass
@@ -259,6 +316,10 @@ class Dataset:
                 is_labeled = True
             except (ValueError, audformat.core.errors.BadKeyError) as e:
                 pass
+            # copy other column
+            for column in source_df.columns:
+                if column not in [self.target, "age", "speaker", "gender"]:
+                    df_local[column] = source_df[column]
             df = pd.concat([df, df_local])
         return df, is_labeled, got_speaker, got_gender, got_age
 
@@ -268,7 +329,7 @@ class Dataset:
         storage_test = f"{store}{self.name}_testdf.pkl"
         storage_train = f"{store}{self.name}_traindf.pkl"
         split_strategy = self.util.config_val_data(
-            self.name, "split_strategy", "database"
+            self.name, "split_strategy", "speaker_split"
         )
         self.util.debug(
             f"splitting database {self.name} with strategy {split_strategy}"
@@ -362,6 +423,19 @@ class Dataset:
             self.util.debug(f"{self.name}: trying to reuse data splits")
             self.df_test = pd.read_pickle(storage_test)
             self.df_train = pd.read_pickle(storage_train)
+        elif isinstance(ast.literal_eval(split_strategy), list):
+            # treat this as a list of test speakers
+            self.assign_speakers(ast.literal_eval(split_strategy))
+        else:
+            self.util.error(f"unknown split strategy: {split_strategy}")
+
+        # check if train or test set should be ignored
+        as_test = eval(self.util.config_val_data(self.name, "as_test", "False"))
+        if as_test:
+            self.df_train = pd.DataFrame()
+        as_train = eval(self.util.config_val_data(self.name, "as_train", "False"))
+        if as_train:
+            self.df_test = pd.DataFrame()
 
         if self.df_test.shape[0] > 0:
             self.df_test = self.finish_up(self.df_test, storage_test)
@@ -386,6 +460,9 @@ class Dataset:
 
     def balanced_split(self):
         """One way to split train and eval sets: Generate split dataframes for some balancing criterion"""
+        from splitutils import binning
+        from splitutils import optimize_traintest_split
+
         seed = 42
         k = 30
         test_size = int(self.util.config_val_data(self.name, "test_size", 20)) / 100.0
@@ -396,7 +473,7 @@ class Dataset:
         bins = self.util.config_val("DATA", f"bin", False)
         if bins:
             nbins = len(ast.literal_eval(bins))
-            targets = split.binning(targets, nbins=nbins)
+            targets = binning(targets, nbins=nbins)
         # on which variable to split
         speakers = df["speaker"].to_numpy()
 
@@ -415,7 +492,7 @@ class Dataset:
                     data = df[stratif_var].to_numpy()
                     bins = self.util.config_val("DATA", f"{stratif_var}_bins", False)
                     if bins:
-                        data = split.binning(data, nbins=int(bins))
+                        data = binning(data, nbins=int(bins))
                     stratif_vars_array[stratif_var] = data
         # weights for all stratify_on variables and
         # and for test proportion match. Give target
@@ -428,7 +505,8 @@ class Dataset:
             weights[key] = value
         # find optimal test indices TEST_I in DF
         # info: dict with goodness of split information
-        train_i, test_i, info = split.optimize_traintest_split(
+
+        train_i, test_i, info = optimize_traintest_split(
             X=df,
             y=targets,
             split_on=speakers,
@@ -448,6 +526,19 @@ class Dataset:
         # because this generates new train/test sample quantaties, the feature extraction has to be done again
         glob_conf.config["FEATS"]["needs_feature_extraction"] = "True"
 
+    def assign_speakers(self, speakers):
+        """One way to split train and eval sets: Specify test speaker names."""
+        self.df_test = self.df[self.df.speaker.isin(speakers)]
+        if len(self.df_test) == 0:
+            self.util.error(f"no speakers found in {speakers}")
+        self.df_train = self.df[~self.df.index.isin(self.df_test.index)]
+        self.util.debug(
+            f"{self.name} (speakers assigned): [{self.df_train.shape[0]}/{self.df_test.shape[0]}]"
+            " samples in train/test"
+        )
+        # because this generates new train/test sample quantaties, the feature extraction has to be done again
+        glob_conf.config["FEATS"]["needs_feature_extraction"] = "True"
+
     def split_speakers(self):
         """One way to split train and eval sets: Specify percentage of evaluation speakers"""
         test_percent = int(self.util.config_val_data(self.name, "test_size", 20))
@@ -490,21 +581,22 @@ class Dataset:
     def prepare_labels(self):
         # strategy = self.util.config_val("DATA", "strategy", "train_test")
         only_tests = eval(self.util.config_val("DATA", "tests", "False"))
-        # if strategy == "cross_data" or only_tests:
-        #     self.df = self.map_labels(self.df)
-        #     # Bin target values if they are continuous but a classification experiment should be done
-        #     self.map_continuous_classification(self.df)
-        #     self.df = self._add_labels(self.df)
-        #     if self.util.config_val_data(self.name, "value_counts", False):
-        #         if not self.got_gender or not self.got_speaker:
-        #             self.util.error(
-        #                 "can't plot value counts if no speaker or gender is" " given"
-        #             )
-        #         else:
-        #             self.plot.describe_df(
-        #                 self.name, self.df, self.target, f"{self.name}_distplot"
-        #             )
-        # elif strategy == "train_test":
+        module = glob_conf.module
+        if only_tests and module == "test":
+            self.df_test = self.map_labels(self.df_test)
+            # Bin target values if they are continuous but a classification experiment should be done
+            self.map_continuous_classification(self.df_test)
+            self.df_test = self._add_labels(self.df_test)
+            if self.util.config_val_data(self.name, "value_counts", False):
+                if not self.got_gender or not self.got_speaker:
+                    self.util.error(
+                        "can't plot value counts if no speaker or gender is" " given"
+                    )
+                else:
+                    self.plot.describe_df(
+                        self.name, self.df_test, self.target, f"{self.name}_distplot"
+                    )
+            return
         self.df_train = self.map_labels(self.df_train)
         self.df_test = self.map_labels(self.df_test)
         self.map_continuous_classification(self.df_train)
@@ -557,7 +649,7 @@ class Dataset:
         return df
 
     def check_continuous_classification(self):
-        datatype = self.util.config_val("DATA", "type", "False")
+        datatype = self.util.config_val("DATA", "type", False)
         if self.util.exp_is_classification() and datatype == "continuous":
             return True
         return False
@@ -567,7 +659,7 @@ class Dataset:
         if self.check_continuous_classification():
             self.util.debug(f"{self.name}: binning continuous variable to categories")
             cat_vals = self.util.continuous_to_categorical(df[self.target])
-            df[self.target] = cat_vals
+            df[self.target] = cat_vals.values
             labels = ast.literal_eval(glob_conf.config["DATA"]["labels"])
             df["class_label"] = df[self.target]
             for i, l in enumerate(labels):
