@@ -4,51 +4,55 @@ import os
 import pickle
 import random
 import time
-from cgi import print_environ  # To convert strings to objects
 
-import audeer
-import audformat
 import numpy as np
 import pandas as pd
 from sklearn.preprocessing import LabelEncoder
 
-import nkululeko.glob_conf as glob_conf
+import audeer
+import audformat
+
 from nkululeko.data.dataset import Dataset
 from nkululeko.data.dataset_csv import Dataset_CSV
 from nkululeko.demo_predictor import Demo_predictor
 from nkululeko.feat_extract.feats_analyser import FeatureAnalyser
 from nkululeko.feature_extractor import FeatureExtractor
 from nkululeko.file_checker import FileChecker
-from nkululeko.filter_data import DataFilter, filter_min_dur
+from nkululeko.filter_data import DataFilter
+import nkululeko.glob_conf as glob_conf
 from nkululeko.plots import Plots
 from nkululeko.reporting.report import Report
 from nkululeko.runmanager import Runmanager
 from nkululeko.scaler import Scaler
-from nkululeko.test_predictor import Test_predictor
-from nkululeko.util import Util
+from nkululeko.test_predictor import TestPredictor
+from nkululeko.utils.util import Util
 
 
 class Experiment:
-    """Main class specifying an experiment"""
+    """Main class specifying an experiment."""
 
     def __init__(self, config_obj):
-        """
-        Parameters
-        ----------
-        config_obj : a config parser object that sets the experiment parameters and being set as a global object.
-        """
+        """Constructor.
 
+        Args:
+            - config_obj : a config parser object that sets the experiment parameters and being set as a global object.
+        """
         self.set_globals(config_obj)
         self.name = glob_conf.config["EXP"]["name"]
-        self.root = glob_conf.config["EXP"]["root"]
+        self.root = os.path.join(glob_conf.config["EXP"]["root"], "")
         self.data_dir = os.path.join(self.root, self.name)
         audeer.mkdir(self.data_dir)  # create the experiment directory
         self.util = Util("experiment")
         glob_conf.set_util(self.util)
-        try:
-            with open(os.path.join(self.data_dir, "report.pkl"), "rb") as handle:
-                self.report = pickle.load(handle)
-        except FileNotFoundError:
+        fresh_report = eval(self.util.config_val("REPORT", "fresh", "False"))
+        if not fresh_report:
+            try:
+                with open(os.path.join(self.data_dir, "report.pkl"), "rb") as handle:
+                    self.report = pickle.load(handle)
+            except FileNotFoundError:
+                self.report = Report()
+        else:
+            self.util.debug("starting a fresh report")
             self.report = Report()
         glob_conf.set_report(self.report)
         self.loso = self.util.config_val("MODEL", "loso", False)
@@ -56,16 +60,20 @@ class Experiment:
         self.xfoldx = self.util.config_val("MODEL", "k_fold_cross", False)
         self.start = time.process_time()
 
+    def set_module(self, module):
+        glob_conf.set_module(module)
+
     def store_report(self):
         with open(os.path.join(self.data_dir, "report.pkl"), "wb") as handle:
             pickle.dump(self.report, handle)
         if eval(self.util.config_val("REPORT", "show", "False")):
             self.report.print()
         if self.util.config_val("REPORT", "latex", False):
-            self.report.export_latex() 
+            self.report.export_latex()
 
-    def get_name(self):
-        return self.util.get_exp_name()
+    # moved to util
+    # def get_name(self):
+    #     return self.util.get_exp_name()
 
     def set_globals(self, config_obj):
         """install a config object in the global space"""
@@ -94,20 +102,21 @@ class Experiment:
                 self.got_speaker = True
             self.datasets.update({d: data})
         self.target = self.util.config_val("DATA", "target", "emotion")
+        glob_conf.set_target(self.target)
         # print target via debug
         self.util.debug(f"target: {self.target}")
         # print keys/column
         dbs = ",".join(list(self.datasets.keys()))
         labels = self.util.config_val("DATA", "labels", False)
+        auto_labels = list(next(iter(self.datasets.values())).df[self.target].unique())
         if labels:
-            labels = ast.literal_eval(labels)
+            self.labels = ast.literal_eval(labels)
+            self.util.debug(f"Using target labels (from config): {labels}")
         else:
-            labels = list(
-                next(iter(self.datasets.values())).df[self.target].unique()
-            )
-        # print labels via debug
-        self.util.debug(f"Target labels (user defined): {labels}")
-        glob_conf.set_labels(labels)
+            self.labels = auto_labels
+        # print autolabel no matter it is specified or not
+        self.util.debug(f"Labels (from database): {auto_labels}")
+        glob_conf.set_labels(self.labels)
         self.util.debug(f"loaded databases {dbs}")
 
     def _import_csv(self, storage):
@@ -146,16 +155,17 @@ class Experiment:
                     self.got_age = True
                 if data.got_speaker:
                     self.got_speaker = True
+                data.split()
                 data.prepare_labels()
                 self.df_test = pd.concat(
-                    [self.df_test, self.util.make_segmented_index(data.df)]
+                    [self.df_test, self.util.make_segmented_index(data.df_test)]
                 )
                 self.df_test.is_labeled = data.is_labeled
             self.df_test.got_gender = self.got_gender
             self.df_test.got_speaker = self.got_speaker
             # self.util.set_config_val('FEATS', 'needs_features_extraction', 'True')
             # self.util.set_config_val('FEATS', 'no_reuse', 'True')
-            self.df_test["class_labels"] = self.df_test[self.target]
+            self.df_test["class_label"] = self.df_test[self.target]
             self.df_test[self.target] = self.label_encoder.transform(
                 self.df_test[self.target]
             )
@@ -176,9 +186,7 @@ class Experiment:
                 f"reusing previously stored {storage_test} and {storage_train}"
             )
             self.df_test = self._import_csv(storage_test)
-            # print(f"df_test: {self.df_test}")
             self.df_train = self._import_csv(storage_train)
-            # print(f"df_train: {self.df_train}")
         else:
             self.df_train, self.df_test = pd.DataFrame(), pd.DataFrame()
             for d in self.datasets.values():
@@ -232,24 +240,24 @@ class Experiment:
         if self.util.exp_is_classification():
             datatype = self.util.config_val("DATA", "type", "dummy")
             if datatype == "continuous":
-                if self.df_test.is_labeled:
-                    # remember the target in case they get labelencoded later
-                    self.df_test["class_label"] = self.df_test[self.target]
-                    test_cats = self.df_test["class_label"].unique()
-                else:
-                    # if there is no target, copy a dummy label
-                    self.df_test = self._add_random_target(self.df_test)
-                if self.df_train.is_labeled:
-                    # remember the target in case they get labelencoded later
-                    self.df_train["class_label"] = self.df_train[self.target]
-                    train_cats = self.df_train["class_label"].unique()
+                # if self.df_test.is_labeled:
+                #     # remember the target in case they get labelencoded later
+                #     self.df_test["class_label"] = self.df_test[self.target]
+                test_cats = self.df_test["class_label"].unique()
+                # else:
+                #     # if there is no target, copy a dummy label
+                #     self.df_test = self._add_random_target(self.df_test)
+                # if self.df_train.is_labeled:
+                #     # remember the target in case they get labelencoded later
+                #     self.df_train["class_label"] = self.df_train[self.target]
+                train_cats = self.df_train["class_label"].unique()
+
             else:
                 if self.df_test.is_labeled:
                     test_cats = self.df_test[self.target].unique()
                 else:
                     # if there is no target, copy a dummy label
-                    self.df_test = self._add_random_target(
-                        self.df_test).astype('str')
+                    self.df_test = self._add_random_target(self.df_test).astype("str")
                 train_cats = self.df_train[self.target].unique()
                 # print(f"df_train: {pd.DataFrame(self.df_train[self.target])}")
                 # print(f"train_cats with target {self.target}: {train_cats}")
@@ -257,11 +265,11 @@ class Experiment:
                 if type(test_cats) == np.ndarray:
                     self.util.debug(f"Categories test (nd.array): {test_cats}")
                 else:
-                    self.util.debug(f"Categories test (list): {test_cats.to_list()}")
+                    self.util.debug(f"Categories test (list): {list(test_cats)}")
             if type(train_cats) == np.ndarray:
                 self.util.debug(f"Categories train (nd.array): {train_cats}")
             else:
-                self.util.debug(f"Categories train (list): {train_cats.to_list()}")
+                self.util.debug(f"Categories train (list): {list(train_cats)}")
 
             # encode the labels as numbers
             self.label_encoder = LabelEncoder()
@@ -280,19 +288,17 @@ class Experiment:
 
         target_factor = self.util.config_val("DATA", "target_divide_by", False)
         if target_factor:
-            self.df_test[self.target] = self.df_test[self.target] / float(
-                target_factor
-            )
+            self.df_test[self.target] = self.df_test[self.target] / float(target_factor)
             self.df_train[self.target] = self.df_train[self.target] / float(
                 target_factor
             )
             if not self.util.exp_is_classification():
-                self.df_test["class_label"] = self.df_test[
-                    "class_label"
-                ] / float(target_factor)
-                self.df_train["class_label"] = self.df_train[
-                    "class_label"
-                ] / float(target_factor)
+                self.df_test["class_label"] = self.df_test["class_label"] / float(
+                    target_factor
+                )
+                self.df_train["class_label"] = self.df_train["class_label"] / float(
+                    target_factor
+                )
 
     def _add_random_target(self, df):
         labels = glob_conf.labels
@@ -305,18 +311,14 @@ class Experiment:
     def plot_distribution(self, df_labels):
         """Plot the distribution of samples and speaker per target class and biological sex"""
         plot = Plots()
-        sample_selection = self.util.config_val(
-            "EXPL", "sample_selection", "all"
-        )
+        sample_selection = self.util.config_val("EXPL", "sample_selection", "all")
         plot.plot_distributions(df_labels)
         if self.got_speaker:
             plot.plot_distributions_speaker(df_labels)
 
     def extract_test_feats(self):
         self.feats_test = pd.DataFrame()
-        feats_name = "_".join(
-            ast.literal_eval(glob_conf.config["DATA"]["tests"])
-        )
+        feats_name = "_".join(ast.literal_eval(glob_conf.config["DATA"]["tests"]))
         feats_types = self.util.config_val_list("FEATS", "type", ["os"])
         self.feature_extractor = FeatureExtractor(
             self.df_test, feats_types, feats_name, "test"
@@ -333,11 +335,21 @@ class Experiment:
 
         """
         df_train, df_test = self.df_train, self.df_test
-        feats_name = "_".join(
-            ast.literal_eval(glob_conf.config["DATA"]["databases"])
-        )
+        feats_name = "_".join(ast.literal_eval(glob_conf.config["DATA"]["databases"]))
         self.feats_test, self.feats_train = pd.DataFrame(), pd.DataFrame()
-        feats_types = self.util.config_val_list("FEATS", "type", ["os"])
+        feats_types = self.util.config_val("FEATS", "type", "os")
+        # Ensure feats_types is always a list of strings
+        if isinstance(feats_types, str):
+            if feats_types.startswith("[") and feats_types.endswith("]"):
+                feats_types = ast.literal_eval(feats_types)
+            else:
+                feats_types = [feats_types]
+        # print(f"feats_types: {feats_types}")
+        # for some models no features are needed
+        if len(feats_types) == 0:
+            self.util.debug("no feature extractor specified.")
+            self.feats_train, self.feats_test = pd.DataFrame(), pd.DataFrame()
+            return
         self.feature_extractor = FeatureExtractor(
             df_train, feats_types, feats_name, "train"
         )
@@ -358,26 +370,28 @@ class Experiment:
             self.df_train = self.df_train[
                 self.df_train.index.isin(self.feats_train.index)
             ]
-            self.util.warn(f"mew train labels shape: {self.df_train.shape[0]}")
+            self.util.warn(f"new train labels shape: {self.df_train.shape[0]}")
         if self.feats_test.shape[0] < self.df_test.shape[0]:
             self.util.warn(
                 f"test feats ({self.feats_test.shape[0]}) != test labels"
                 f" ({self.df_test.shape[0]})"
             )
-            self.df_test = self.df_test[
-                self.df_test.index.isin(self.feats_test.index)
-            ]
-            self.util.warn(f"mew test labels shape: {self.df_test.shape[0]}")
+            self.df_test = self.df_test[self.df_test.index.isin(self.feats_test.index)]
+            self.util.warn(f"new test labels shape: {self.df_test.shape[0]}")
 
         self._check_scale()
+        # store = self.util.get_path("store")
+        # store_format = self.util.config_val("FEATS", "store_format", "pkl")
+        # storage = f"{store}test_feats.{store_format}"
+        # self.util.write_store(self.feats_test, storage, store_format)
+        # storage = f"{store}train_feats.{store_format}"
+        # self.util.write_store(self.feats_train, storage, store_format)
 
     def augment(self):
-        """
-        Augment the selected samples
-        """
+        """Augment the selected samples."""
         from nkululeko.augmenting.augmenter import Augmenter
 
-        sample_selection = self.util.config_val("DATA", "augment", "train")
+        sample_selection = self.util.config_val("AUGMENT", "sample_selection", "all")
         if sample_selection == "all":
             df = pd.concat([self.df_train, self.df_test])
         elif sample_selection == "train":
@@ -391,7 +405,8 @@ class Experiment:
             )
 
         augmenter = Augmenter(df)
-        augmenter.augment(sample_selection)
+        df_ret = augmenter.augment(sample_selection)
+        return df_ret
 
     def autopredict(self):
         """
@@ -457,8 +472,7 @@ class Experiment:
                 predictor = ValencePredictor(df)
                 df = predictor.predict(sample_selection)
             elif target == "dominance":
-                from nkululeko.autopredict.ap_dominance import \
-                    DominancePredictor
+                from nkululeko.autopredict.ap_dominance import DominancePredictor
 
                 predictor = DominancePredictor(df)
                 df = predictor.predict(sample_selection)
@@ -472,9 +486,7 @@ class Experiment:
         """
         from nkululeko.augmenting.randomsplicer import Randomsplicer
 
-        sample_selection = self.util.config_val(
-            "DATA", "random_splice", "train"
-        )
+        sample_selection = self.util.config_val("AUGMENT", "sample_selection", "all")
         if sample_selection == "all":
             df = pd.concat([self.df_train, self.df_test])
         elif sample_selection == "train":
@@ -487,20 +499,15 @@ class Experiment:
                 " should be [all | train | test]"
             )
         randomsplicer = Randomsplicer(df)
-        randomsplicer.run(sample_selection)
+        df_ret = randomsplicer.run(sample_selection)
+        return df_ret
 
     def analyse_features(self, needs_feats):
-        """
-        Do a feature exploration
-
-        """
-
+        """Do a feature exploration."""
         plot_feats = eval(
             self.util.config_val("EXPL", "feature_distributions", "False")
         )
-        sample_selection = self.util.config_val(
-            "EXPL", "sample_selection", "all"
-        )
+        sample_selection = self.util.config_val("EXPL", "sample_selection", "all")
         # get the data labels
         if sample_selection == "all":
             df_labels = pd.concat([self.df_train, self.df_test])
@@ -516,15 +523,16 @@ class Experiment:
                 f"unknown sample selection specifier {sample_selection}, should"
                 " be [all | train | test]"
             )
-
+        self.util.debug(f"sampling selection: {sample_selection}")
         if self.util.config_val("EXPL", "value_counts", False):
             self.plot_distribution(df_labels)
 
         # check if data should be shown with the spotlight data visualizer
         spotlight = eval(self.util.config_val("EXPL", "spotlight", "False"))
         if spotlight:
-            self.util.debug('opening spotlight tab in web browser')
+            self.util.debug("opening spotlight tab in web browser")
             from renumics import spotlight
+
             spotlight.show(df_labels.reset_index())
 
         if not needs_feats:
@@ -541,54 +549,59 @@ class Experiment:
                 f"unknown sample selection specifier {sample_selection}, should"
                 " be [all | train | test]"
             )
+        feat_analyser = FeatureAnalyser(sample_selection, df_labels, df_feats)
+        # check if SHAP features should be analysed
+        shap = eval(self.util.config_val("EXPL", "shap", "False"))
+        if shap:
+            feat_analyser.analyse_shap(self.runmgr.get_best_model())
 
         if plot_feats:
-            feat_analyser = FeatureAnalyser(
-                sample_selection, df_labels, df_feats
-            )
             feat_analyser.analyse()
 
         # check if a scatterplot should be done
         scatter_var = eval(self.util.config_val("EXPL", "scatter", "False"))
+        scatter_target = self.util.config_val(
+            "EXPL", "scatter.target", "['class_label']"
+        )
         if scatter_var:
             scatters = ast.literal_eval(glob_conf.config["EXPL"]["scatter"])
-            if self.util.exp_is_classification():
-                plots = Plots()
-                for scatter in scatters:
-                    plots.scatter_plot(
-                        df_feats, df_labels["class_label"], scatter
+            scat_targets = ast.literal_eval(scatter_target)
+            plots = Plots()
+            for scat_target in scat_targets:
+                if self.util.is_categorical(df_labels[scat_target]):
+                    for scatter in scatters:
+                        plots.scatter_plot(df_feats, df_labels, scat_target, scatter)
+                else:
+                    self.util.debug(
+                        f"{self.name}: binning continuous variable to categories"
                     )
-            else:
-                self.util.debug("can't do scatterplot if not classification")
-
-        # check if a tsne should be plotted
-        tsne = eval(self.util.config_val("EXPL", "tsne", "False"))
-        if tsne:
-            if self.util.exp_is_classification():
-                plots = Plots()
-                all_feats = self.feats_train.append(self.feats_test)
-                all_labels = self.df_train["class_label"].append(
-                    self.df_test["class_label"]
-                )
-                plots.plotTsne(
-                    all_feats, all_labels, self.util.get_exp_name() + "_tsne"
-                )
-            else:
-                self.util.debug("can't plot tsne if not classification")
+                    cat_vals = self.util.continuous_to_categorical(
+                        df_labels[scat_target]
+                    )
+                    df_labels[f"{scat_target}_bins"] = cat_vals.values
+                    for scatter in scatters:
+                        plots.scatter_plot(
+                            df_feats, df_labels, f"{scat_target}_bins", scatter
+                        )
 
     def _check_scale(self):
-        scale = self.util.config_val("FEATS", "scale", False)
+        self.util.save_to_store(self.feats_train, "feats_train")
+        self.util.save_to_store(self.feats_test, "feats_test")
+        scale_feats = self.util.config_val("FEATS", "scale", False)
         # print the scale
-        self.util.debug(f"scaler: {scale}")
-        if scale:
-            self.scaler = Scaler(
+        self.util.debug(f"scaler: {scale_feats}")
+        if scale_feats:
+            self.scaler_feats = Scaler(
                 self.df_train,
                 self.df_test,
                 self.feats_train,
                 self.feats_test,
-                scale,
+                scale_feats,
             )
-            self.feats_train, self.feats_test = self.scaler.scale()
+            self.feats_train, self.feats_test = self.scaler_feats.scale()
+            # store versions
+            self.util.save_to_store(self.feats_train, "feats_train_scaled")
+            self.util.save_to_store(self.feats_test, "feats_test_scaled")
 
     def init_runmanager(self):
         """Initialize the manager object for the runs."""
@@ -602,11 +615,13 @@ class Experiment:
 
         # access the best results all runs
         self.reports = self.runmgr.best_results
+        last_epochs = self.runmgr.last_epochs
         # try to save yourself
         save = self.util.config_val("EXP", "save", False)
         if save:
             # save the experiment for future use
             self.save(self.util.get_save_name())
+            # self.save_onnx(self.util.get_save_name())
 
         # self.__collect_reports()
         self.util.print_best_results(self.reports)
@@ -631,70 +646,101 @@ class Experiment:
         if label_data and label_result:
             self.predict_test_and_save(label_result)
 
-        return self.reports
+        return self.reports, last_epochs
 
     def plot_confmat_per_speaker(self, function):
         if self.loso or self.logo or self.xfoldx:
             self.util.debug(
-                "plot combined speaker predictions not possible for cross"
-                " validation"
+                "plot combined speaker predictions not possible for cross" " validation"
             )
             return
-        best = self._get_best_report(self.reports)
-        # if not best.is_classification:
-        #     best.continuous_to_categorical()
-        truths = best.truths
-        preds = best.preds
+        best = self.get_best_report(self.reports)
+        if best.is_classification:
+            truths = best.truths
+            preds = best.preds
+        else:
+            truths = best.truths_cont
+            preds = best.preds_cont
         speakers = self.df_test.speaker.values
-        print(f"{len(truths)} {len(preds)} {len(speakers) }")
-        df = pd.DataFrame(
-            data={"truth": truths, "pred": preds, "speaker": speakers}
-        )
-        plot_name = "result_combined_per_speaker"
+        df = pd.DataFrame(data={"truths": truths, "preds": preds, "speakers": speakers})
+        plot_name = f"{self.util.get_exp_name()}_speakercombined_{function}"
         self.util.debug(
             f"plotting speaker combination ({function}) confusion matrix to"
             f" {plot_name}"
         )
         best.plot_per_speaker(df, plot_name, function)
 
-    def _get_best_report(self, best_reports):
-        return self.runmgr.get_best_result(best_reports)
+    def get_best_report(self, reports):
+        return self.runmgr.get_best_result(reports)
 
     def print_best_model(self):
         self.runmgr.print_best_result_runs()
 
-    def demo(self, file, is_list):
+    def demo(self, file, is_list, outfile):
         model = self.runmgr.get_best_model()
-        labelEncoder = None
+        lab_enc = None
         try:
-            labelEncoder = self.label_encoder
+            lab_enc = self.label_encoder
         except AttributeError:
             pass
         demo = Demo_predictor(
-            model, file, is_list, self.feature_extractor, labelEncoder
+            model, file, is_list, self.feature_extractor, lab_enc, outfile
         )
         demo.run_demo()
 
     def predict_test_and_save(self, result_name):
         model = self.runmgr.get_best_model()
         model.set_testdata(self.df_test, self.feats_test)
-        test_predictor = Test_predictor(
+        test_predictor = TestPredictor(
             model, self.df_test, self.label_encoder, result_name
         )
-        test_predictor.predict_and_store()
+        result = test_predictor.predict_and_store()
+        return result
 
     def load(self, filename):
-        f = open(filename, "rb")
-        tmp_dict = pickle.load(f)
-        f.close()
+        try:
+            f = open(filename, "rb")
+            tmp_dict = pickle.load(f)
+            f.close()
+        except EOFError as eof:
+            self.util.error(f"can't open file {filename}: {eof}")
         self.__dict__.update(tmp_dict)
+        glob_conf.set_labels(self.labels)
 
     def save(self, filename):
+        if self.runmgr.modelrunner.model.is_ann():
+            self.runmgr.modelrunner.model = None
+            self.util.warn(
+                "Save experiment: Can't pickle the trained model so saving without it. (it should be stored anyway)"
+            )
         try:
             f = open(filename, "wb")
             pickle.dump(self.__dict__, f)
             f.close()
-        except (AttributeError, TypeError, RuntimeError) as error:
+        except (TypeError, AttributeError) as error:
+            self.feature_extractor.feat_extractor.model = None
+            f = open(filename, "wb")
+            pickle.dump(self.__dict__, f)
+            f.close()
             self.util.warn(
-                f"Save experiment: Can't pickle local object: {error}"
+                "Save experiment: Can't pickle the feature extraction model so saving without it."
+                + f"{type(error).__name__} {error}"
             )
+        except RuntimeError as error:
+            self.util.warn(
+                "Save experiment: Can't pickle local object, NOT saving: "
+                + f"{type(error).__name__} {error}"
+            )
+
+    def save_onnx(self, filename):
+        # export the model to onnx
+        model = self.runmgr.get_best_model()
+        if model.is_ann():
+            print("converting to onnx from torch")
+        else:
+
+            print("converting to onnx from sklearn")
+        # save the rest
+        f = open(filename, "wb")
+        pickle.dump(self.__dict__, f)
+        f.close()
