@@ -16,6 +16,11 @@ def set_util(util_obj):
     util = util_obj
 
 
+def set_module(module_obj):
+    global module
+    module = module_obj
+
+
 def set_report(report_obj):
     global report
     report = report_obj
@@ -24,3 +29,8 @@ def set_report(report_obj):
 def set_labels(labels_obj):
     global labels
     labels = labels_obj
+
+
+def set_target(target_obj):
+    global target
+    target = target_obj
