@@ -11,8 +11,11 @@ class SVR_model(Model):
 
     def __init__(self, df_train, df_test, feats_train, feats_test):
         super().__init__(df_train, df_test, feats_train, feats_test)
+        self.name = "svr"
         c = float(self.util.config_val("MODEL", "C_val", "0.001"))
-        self.clf = svm.SVR(kernel="rbf", C=c)  # set up the classifier
+        #  kernel{‘linear’, ‘poly’, ‘rbf’, ‘sigmoid’, ‘precomputed’} or callable, default=’rbf’
+        kernel = self.util.config_val("MODEL", "kernel", "rbf")
+        self.clf = svm.SVR(kernel=kernel, C=c)  # set up the classifier
 
     def set_C(self, c):
         """Set the C parameter"""
