@@ -1,27 +1,17 @@
 # nkululeko.py
 # Entry script to do a Nkululeko experiment
-
-import os.path
-import configparser
 import argparse
-import nkululeko.experiment as exp
-from nkululeko.util import Util
+import configparser
+import os.path
+
+import numpy as np
+
 from nkululeko.constants import VERSION
+import nkululeko.experiment as exp
+from nkululeko.utils.util import Util
 
 
-def main(src_dir):
-    parser = argparse.ArgumentParser(
-        description="Call the nkululeko framework."
-    )
-    parser.add_argument(
-        "--config", default="exp.ini", help="The base configuration"
-    )
-    args = parser.parse_args()
-    if args.config is not None:
-        config_file = args.config
-    else:
-        config_file = f"{src_dir}/exp.ini"
-
+def doit(config_file):
     # test if the configuration file exists
     if not os.path.isfile(config_file):
         print(f"ERROR: no such file: {config_file}")
@@ -33,7 +23,9 @@ def main(src_dir):
 
     # create a new experiment
     expr = exp.Experiment(config)
-    util = Util("nkululeko")
+    module = "nkululeko"
+    expr.set_module(module)
+    util = Util(module)
     util.debug(
         f"running {expr.name} from config {config_file}, nkululeko version"
         f" {VERSION}"
@@ -49,9 +41,7 @@ def main(src_dir):
 
     # split into train and test
     expr.fill_train_and_tests()
-    util.debug(
-        f"train shape : {expr.df_train.shape}, test shape:{expr.df_test.shape}"
-    )
+    util.debug(f"train shape : {expr.df_train.shape}, test shape:{expr.df_test.shape}")
 
     # extract features
     expr.extract_feats()
@@ -60,15 +50,24 @@ def main(src_dir):
     expr.init_runmanager()
 
     # run the experiment
-    expr.run()
-
+    reports, last_epochs = expr.run()
+    result = expr.get_best_report(reports).result.test
     expr.store_report()
-
     print("DONE")
+    return result, int(np.asarray(last_epochs).min())
+
+
+def main(src_dir):
+    parser = argparse.ArgumentParser(description="Call the nkululeko framework.")
+    parser.add_argument("--config", default="exp.ini", help="The base configuration")
+    args = parser.parse_args()
+    if args.config is not None:
+        config_file = args.config
+    else:
+        config_file = f"{src_dir}/exp.ini"
+    doit(config_file)
 
 
 if __name__ == "__main__":
     cwd = os.path.dirname(os.path.abspath(__file__))
-    main(
-        cwd
-    )  # use this if you want to state the config file path on command line
+    main(cwd)  # use this if you want to state the config file path on command line
