@@ -4,10 +4,11 @@ report.py
 Collector class for report items collected during module processing.
 
 """
-from nkululeko.util import Util
+from nkululeko.utils.util import Util
 from nkululeko.reporting.report_item import ReportItem
 from nkululeko.reporting.latex_writer import LatexWriter
 
+
 class Report:
     def __init__(self):
         self.report_items = {}
